package SPVM::Mojo::Log::Callback::Format;



1;

=head1 Name

SPVM::Mojo::Log::Callback::Format - Short Description

=head1 Description

Mojo::Log::Callback::Format interface in L<SPVM> has interface methods to do someting.

=head1 Usage

  interface Mojo::Log::Callback::Format;

=head1 Details



=head1 Interface Methods




=head1 Repository



=head1 Author

Yuki Kimoto C<kimoto.yuki@gmail.com>

=head1 Copyright & License

Copyright (c) 2025 Yuki Kimoto

MIT License

