package SPVM::Mojo::UserAgent::CookieJar::Callback::Ignore;



1;

=head1 Name

SPVM::Mojo::UserAgent::CookieJar::Callback::Ignore - Callback for Mojo::UserAgent::CookieJar#ignore field

=head1 Description

Mojo::UserAgent::CookieJar::Callback::Ignore interface in L<SPVM> is the callback for L<Mojo::UserAgent::CookieJar#ignore|SPVM::Mojo::UserAgent::CookieJar/"ignore"> field

=head1 Interface Methods

C<required method : int ($cookie : L<Mojo::Cookie::Response|SPVM::Mojo::Cookie::Response>);>

The callback for L<Mojo::UserAgent::CookieJar#ignore|SPVM::Mojo::UserAgent::CookieJar/"ignore"> field.

=head1 See Also

=over 2

=item * L<Mojo::UserAgent::CookieJar|SPVM::Mojo::UserAgent::CookieJar>

=item * L<SPVM::Mojolicious>

=back

=head1 Copyright & License

Copyright (c) 2025 Yuki Kimoto

MIT License

